# This file contains instance methods that are added to Test::Unit::TestCase.

#--
################################################################################
#                      Copyright (C) 2011 Travis Herrick                       #
################################################################################
#                                                                              #
#                                 \v^V,^!v\^/                                  #
#                                 ~%       %~                                  #
#                                 {  _   _  }                                  #
#                                 (  *   -  )                                  #
#                                 |    /    |                                  #
#                                  \   _,  /                                   #
#                                   \__.__/                                    #
#                                                                              #
################################################################################
# This program is free software: you can redistribute it                       #
# and/or modify it under the terms of the GNU Lesser General Public License    #
# as published by the Free Software Foundation,                                #
# either version 3 of the License, or (at your option) any later version.      #
################################################################################
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY;                                                    #
# without even the implied warranty of MERCHANTABILITY                         #
# or FITNESS FOR A PARTICULAR PURPOSE.                                         #
# See the GNU Lesser General Public License for more details.                  #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.        #
################################################################################
#++

# Namespace for this gem.
module TestUnitHelper
  # Container for methods
  # that are added to Test::Unit::TestCase as instance methods.
  module InstanceMethods
    # Override initialize to add a few things.
    def initialize(*args)
      super

      reset_io

      unless self.class.test_class.nil?
        @class = Kernel.const_get(self.class.test_class)
      end
    end

    ############################################################################
    private
    ############################################################################

    ############################################################################
    # I/O support methods.
    ############################################################################

    # Wrap a block to capture the output on stdout and stderr.
    # ==== Input
    # [Block] A block that will have stdout and stderr trapped while it runs.
    def wrap_output
      begin
        $stdout = @out
        $stderr = @err
        yield
      ensure
        $stdout = STDOUT
        $stderr = STDERR
      end
    end

    # The output from stdout while wrap_output was in effect.
    # ==== Output
    # [String] The output from stdout.
    def out
      @out.respond_to?(:string) ? @out.string.sub(/\n*\z/, '') : ''
    end

    # The output from stderr while wrap_output was in effect.
    # ==== Output
    # [String] The output from stderr.
    def err
      @err.respond_to?(:string) ? @err.string.sub(/\n*\z/, '') : ''
    end

    # Reset the stdout and stderr stream variables.
    def reset_io
      @out = StringIO.new
      @err = StringIO.new
    end

    # The combination of out and err in one function.
    # ==== Output
    # [Array] Two element array of strings.
    #
    #         The first element is from stdout.
    #
    #         The second element is from stderr.
    def real_finis
      return out, err
    end
  end
end
